"use strict";
/**
 * @copyright   2017, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@boost/core");
const mergeWith_1 = __importDefault(require("lodash/mergeWith"));
const optimal_1 = __importStar(require("optimal"));
const constants_1 = require("./constants");
class Driver extends core_1.Plugin {
    constructor(options = {}) {
        super(options);
        this.command = {};
        this.options = optimal_1.default(options, {
            args: optimal_1.array(optimal_1.string()),
            dependencies: optimal_1.array(optimal_1.string()),
            env: optimal_1.object(optimal_1.union([optimal_1.bool(), optimal_1.number(), optimal_1.string()])),
            strategy: optimal_1.string(constants_1.STRATEGY_NATIVE).oneOf([
                constants_1.STRATEGY_NATIVE,
                constants_1.STRATEGY_CREATE,
                constants_1.STRATEGY_REFERENCE,
                constants_1.STRATEGY_COPY,
            ]),
        }, {
            name: this.constructor.name,
        });
    }
    /**
     * Format the configuration file before it's written.
     */
    formatConfig(data) {
        const content = JSON.stringify(data, null, 2);
        if (this.metadata.configName.endsWith('.js')) {
            return `module.exports = ${content};`;
        }
        return content;
    }
    /**
     * Return a list of user defined arguments.
     */
    getArgs() {
        return [...this.options.args];
    }
    /**
     * Return a list of dependent drivers.
     */
    getDependencies() {
        return [
            // Always required; configured by the driver
            ...this.metadata.dependencies,
            // Custom; configured by the consumer
            ...this.options.dependencies,
        ];
    }
    /**
     * Return a list of supported CLI options.
     */
    getSupportedOptions() {
        return [];
    }
    /**
     * Handle command failures according to this driver.
     */
    handleFailure(error) {
        const { stderr, stdout } = error;
        const out = (stderr || stdout).trim();
        // Integration debugging
        // this.tool.logError('STDERR', JSON.stringify(error));
        if (out) {
            this.tool.logError(out);
        }
    }
    /**
     * Handle successful commands according to this driver.
     */
    handleSuccess(response) {
        const out = response.stdout.trim();
        // Integration debugging
        // this.tool.log('STDOUT', JSON.stringify(response));
        if (out) {
            this.tool.log(out);
        }
    }
    /**
     * Special case for merging arrays.
     */
    handleMerge(prevValue, nextValue) {
        if (Array.isArray(prevValue)) {
            return prevValue.concat(nextValue);
        }
        return undefined;
    }
    /**
     * Merge multiple configuration objects.
     */
    mergeConfig(prev, next) {
        return mergeWith_1.default(prev, next, this.handleMerge);
    }
    /**
     * Easily register events in the tool.
     */
    on(eventName, listener) {
        this.tool.on(eventName, listener);
        return this;
    }
    /**
     * Setup additional command options.
     */
    setCommandOptions(options) {
        const blueprint = {};
        Object.keys(options).forEach(key => {
            blueprint[key] = optimal_1.shape({
                alias: optimal_1.union([optimal_1.string(), optimal_1.array(optimal_1.string())], ''),
                description: optimal_1.string().required(),
            });
        });
        this.command = optimal_1.default(options, blueprint, {
            name: this.constructor.name,
        });
        return this;
    }
    /**
     * Set metadata about the binary/executable in which this driver wraps.
     */
    setMetadata(metadata) {
        this.metadata = optimal_1.default(metadata, {
            bin: optimal_1.string()
                .match(/^[a-z0-9-]+$/u)
                .required(),
            configName: optimal_1.string().required(),
            configOption: optimal_1.string('--config').empty(),
            configStrategy: optimal_1.string(constants_1.STRATEGY_CREATE).oneOf([
                constants_1.STRATEGY_CREATE,
                constants_1.STRATEGY_REFERENCE,
                constants_1.STRATEGY_COPY,
            ]),
            dependencies: optimal_1.array(optimal_1.string()),
            description: optimal_1.string().empty(),
            filterOptions: optimal_1.bool(false),
            helpOption: optimal_1.string('--help'),
            title: optimal_1.string().required(),
            useConfigOption: optimal_1.bool(),
            workspaceStrategy: optimal_1.string(constants_1.STRATEGY_REFERENCE).oneOf([constants_1.STRATEGY_REFERENCE, constants_1.STRATEGY_COPY]),
        }, {
            name: this.constructor.name,
        });
        return this;
    }
}
exports.default = Driver;
